@@ -61,8 +61,8 @@ typedef enum glamor_pixmap_type {
 } glamor_pixmap_type_t;
 
 typedef Bool (*GetDrawableModifiersFuncPtr) (DrawablePtr draw,
-                                             CARD32 format,
-                                             CARD32 *num_modifiers,
+                                             uint32_t format,
+                                             uint32_t *num_modifiers,
                                              uint64_t **modifiers);
 
 #define GLAMOR_EGL_EXTERNAL_BUFFER 3
@@ -141,7 +141,7 @@ extern _X_EXPORT void glamor_egl_exchange_buffers(PixmapPtr front,
 extern _X_EXPORT void glamor_pixmap_exchange_fbos(PixmapPtr front,
                                                   PixmapPtr back);
 
-/* The DDX is not supposed to call these three functions */
+/* The DDX is not supposed to call these four functions */
 extern _X_EXPORT void glamor_enable_dri3(ScreenPtr screen);
 extern _X_EXPORT int glamor_egl_fds_from_pixmap(ScreenPtr, PixmapPtr, int *,
                                                 uint32_t *, uint32_t *,
@@ -150,6 +150,7 @@ extern _X_EXPORT int glamor_egl_fd_name_from_pixmap(ScreenPtr, PixmapPtr,
                                                     CARD16 *, CARD32 *);
 
 extern _X_EXPORT struct gbm_device *glamor_egl_get_gbm_device(ScreenPtr screen);
+extern _X_EXPORT int glamor_egl_fd_from_pixmap(ScreenPtr, PixmapPtr, CARD16 *, CARD32 *);
 
 /* @glamor_supports_pixmap_import_export: Returns whether
  * glamor_fds_from_pixmap(), glamor_name_from_pixmap(), and
@@ -264,11 +265,11 @@ extern _X_EXPORT struct gbm_bo *glamor_gbm_bo_from_pixmap(ScreenPtr screen,
  * */
 extern _X_EXPORT PixmapPtr glamor_pixmap_from_fds(ScreenPtr screen,
                                                   CARD8 num_fds,
-                                                  int *fds,
+                                                  const int *fds,
                                                   CARD16 width,
                                                   CARD16 height,
-                                                  CARD32 *strides,
-                                                  CARD32 *offsets,
+                                                  const CARD32 *strides,
+                                                  const CARD32 *offsets,
                                                   CARD8 depth,
                                                   CARD8 bpp,
                                                   uint64_t modifier);
@@ -318,13 +319,13 @@ extern _X_EXPORT Bool glamor_get_formats(ScreenPtr screen,
                                          CARD32 **formats);
 
 extern _X_EXPORT Bool glamor_get_modifiers(ScreenPtr screen,
-                                           CARD32 format,
-                                           CARD32 *num_modifiers,
+                                           uint32_t format,
+                                           uint32_t *num_modifiers,
                                            uint64_t **modifiers);
 
 extern _X_EXPORT Bool glamor_get_drawable_modifiers(DrawablePtr draw,
-                                                    CARD32 format,
-                                                    CARD32 *num_modifiers,
+                                                    uint32_t format,
+                                                    uint32_t *num_modifiers,
                                                     uint64_t **modifiers);
 
 extern _X_EXPORT void glamor_set_drawable_modifiers_func(ScreenPtr screen,
@@ -391,7 +392,10 @@ extern _X_EXPORT Bool glamor_egl_create_textured_pixmap(PixmapPtr pixmap,
  */
 extern _X_EXPORT Bool
  glamor_egl_create_textured_pixmap_from_gbm_bo(PixmapPtr pixmap,
-                                               struct gbm_bo *bo);
+                                               struct gbm_bo *bo,
+                                               Bool used_modifiers);
+
+extern _X_EXPORT const char *glamor_egl_get_driver_name(ScreenPtr screen);
 
 #endif
 
